@@ -97,6 +97,15 @@ impl_from_cql_val!(Uuid, as_uuid); // Uuid::from_cql<CqlValue>
 impl_from_cql_val!(BigDecimal, into_decimal); // BigDecimal::from_cql<CqlValue>
 impl_from_cql_val!(Duration, as_duration); // Duration::from_cql<CqlValue>
 
+impl FromCqlVal<CqlValue> for crate::frame::value::Time {
+    fn from_cql(cql_val: CqlValue) -> Result<Self, FromCqlValError> {
+        match cql_val {
+            CqlValue::Time(d) => Ok(Self(d)),
+            _ => Err(FromCqlValError::BadCqlType),
+        }
+    }
+}
+
 // Vec<T>::from_cql<CqlValue>
 impl<T: FromCqlVal<CqlValue>> FromCqlVal<CqlValue> for Vec<T> {
     fn from_cql(cql_val: CqlValue) -> Result<Self, FromCqlValError> {
@@ -378,7 +387,7 @@ mod tests {
         let min_date: CqlValue = CqlValue::Date(0);
         assert!(NaiveDate::from_cql(min_date).is_err());
 
-        let max_date: CqlValue = CqlValue::Date(u32::max_value());
+        let max_date: CqlValue = CqlValue::Date(u32::MAX);
         assert!(NaiveDate::from_cql(max_date).is_err());
     }
 
@@ -390,11 +399,27 @@ mod tests {
             Duration::from_cql(CqlValue::Time(time_duration)).unwrap(),
         );
 
-        let timestamp_duration = Duration::milliseconds(i64::min_value());
+        let timestamp_duration = Duration::milliseconds(i64::MIN);
         assert_eq!(
             timestamp_duration,
             Duration::from_cql(CqlValue::Timestamp(timestamp_duration)).unwrap(),
         );
+
+        let timestamp_i64 = 997;
+        assert_eq!(
+            timestamp_i64,
+            i64::from_cql(CqlValue::Timestamp(Duration::milliseconds(timestamp_i64))).unwrap()
+        )
+    }
+
+    #[test]
+    fn time_from_cql() {
+        use crate::frame::value::Time;
+        let time_duration = Duration::nanoseconds(86399999999999);
+        assert_eq!(
+            time_duration,
+            Time::from_cql(CqlValue::Time(time_duration)).unwrap().0,
+        );
     }
 
     #[test]
