@@ -1,12 +1,13 @@
 use crate::cql_to_rust::{FromRow, FromRowError};
 use crate::frame::response::event::SchemaChangeEvent;
-use crate::frame::value::Counter;
+use crate::frame::types::vint_decode;
+use crate::frame::value::{Counter, CqlDuration};
 use crate::frame::{frame_errors::ParseError, types};
 use bigdecimal::BigDecimal;
 use byteorder::{BigEndian, ReadBytesExt};
 use bytes::{Buf, Bytes};
+use chrono;
 use chrono::prelude::*;
-use chrono::Duration;
 use num_bigint::BigInt;
 use std::{
     convert::{TryFrom, TryInto},
@@ -41,6 +42,7 @@ pub struct TableSpec {
 
 #[derive(Debug, Clone)]
 pub enum ColumnType {
+    Custom(String),
     Ascii,
     Boolean,
     Blob,
@@ -48,6 +50,7 @@ pub enum ColumnType {
     Date,
     Decimal,
     Double,
+    Duration,
     Float,
     Int,
     BigInt,
@@ -82,13 +85,14 @@ pub enum CqlValue {
     /// Can be converted to chrono::NaiveDate (-262145-1-1 to 262143-12-31) using as_date
     Date(u32),
     Double(f64),
+    Duration(CqlDuration),
     Empty,
     Float(f32),
     Int(i32),
     BigInt(i64),
     Text(String),
     /// Milliseconds since unix epoch
-    Timestamp(Duration),
+    Timestamp(chrono::Duration),
     Inet(IpAddr),
     List(Vec<CqlValue>),
     Map(Vec<(CqlValue, CqlValue)>),
@@ -104,7 +108,7 @@ pub enum CqlValue {
     SmallInt(i16),
     TinyInt(i8),
     /// Nanoseconds since midnight
-    Time(Duration),
+    Time(chrono::Duration),
     Timeuuid(Uuid),
     Tuple(Vec<Option<CqlValue>>),
     Uuid(Uuid),
@@ -128,12 +132,13 @@ impl CqlValue {
 
         // date_days is u32 then converted to i64
         // then we substract 2^31 - this can't panic
-        let days_since_epoch = Duration::days(date_days.into()) - Duration::days(1 << 31);
+        let days_since_epoch =
+            chrono::Duration::days(date_days.into()) - chrono::Duration::days(1 << 31);
 
         NaiveDate::from_ymd(1970, 1, 1).checked_add_signed(days_since_epoch)
     }
 
-    pub fn as_duration(&self) -> Option<Duration> {
+    pub fn as_duration(&self) -> Option<chrono::Duration> {
         match self {
             Self::Timestamp(i) => Some(*i),
             Self::Time(i) => Some(*i),
@@ -141,6 +146,13 @@ impl CqlValue {
         }
     }
 
+    pub fn as_cql_duration(&self) -> Option<CqlDuration> {
+        match self {
+            Self::Duration(i) => Some(*i),
+            _ => None,
+        }
+    }
+
     pub fn as_counter(&self) -> Option<Counter> {
         match self {
             Self::Counter(i) => Some(*i),
@@ -187,6 +199,7 @@ impl CqlValue {
     pub fn as_bigint(&self) -> Option<i64> {
         match self {
             Self::BigInt(i) => Some(*i),
+            Self::Timestamp(d) => Some(d.num_milliseconds()),
             _ => None,
         }
     }
@@ -376,6 +389,13 @@ fn deser_type(buf: &mut &[u8]) -> StdResult<ColumnType, ParseError> {
     use ColumnType::*;
     let id = types::read_short(buf)?;
     Ok(match id {
+        0x0000 => {
+            let type_str: String = types::read_string(buf)?.to_string();
+            match type_str.as_str() {
+                "org.apache.cassandra.db.marshal.DurationType" => Duration,
+                _ => Custom(type_str),
+            }
+        }
         0x0001 => Ascii,
         0x0002 => BigInt,
         0x0003 => Blob,
@@ -395,6 +415,7 @@ fn deser_type(buf: &mut &[u8]) -> StdResult<ColumnType, ParseError> {
         0x0012 => Time,
         0x0013 => SmallInt,
         0x0014 => TinyInt,
+        0x0015 => Duration,
         0x0020 => List(Box::new(deser_type(buf)?)),
         0x0021 => Map(Box::new(deser_type(buf)?), Box::new(deser_type(buf)?)),
         0x0022 => Set(Box::new(deser_type(buf)?)),
@@ -534,6 +555,12 @@ fn deser_cql_value(typ: &ColumnType, buf: &mut &[u8]) -> StdResult<CqlValue, Par
     }
 
     Ok(match typ {
+        Custom(type_str) => {
+            return Err(ParseError::BadData(format!(
+                "Support for custom types is not yet implemented: {}",
+                type_str
+            )));
+        }
         Ascii => {
             if !buf.is_ascii() {
                 return Err(ParseError::BadData("String is not ascii!".to_string()));
@@ -642,7 +669,7 @@ fn deser_cql_value(typ: &ColumnType, buf: &mut &[u8]) -> StdResult<CqlValue, Par
             }
             let millis = buf.read_i64::<BigEndian>()?;
 
-            CqlValue::Timestamp(Duration::milliseconds(millis))
+            CqlValue::Timestamp(chrono::Duration::milliseconds(millis))
         }
         Time => {
             if buf.len() != 8 {
@@ -660,7 +687,7 @@ fn deser_cql_value(typ: &ColumnType, buf: &mut &[u8]) -> StdResult<CqlValue, Par
                 }));
             }
 
-            CqlValue::Time(Duration::nanoseconds(nanoseconds))
+            CqlValue::Time(chrono::Duration::nanoseconds(nanoseconds))
         }
         Timeuuid => {
             if buf.len() != 16 {
@@ -672,6 +699,17 @@ fn deser_cql_value(typ: &ColumnType, buf: &mut &[u8]) -> StdResult<CqlValue, Par
             let uuid = uuid::Uuid::from_slice(buf).expect("Deserializing Uuid failed.");
             CqlValue::Timeuuid(uuid)
         }
+        Duration => {
+            let months = i32::try_from(vint_decode(buf)?)?;
+            let days = i32::try_from(vint_decode(buf)?)?;
+            let nanoseconds = vint_decode(buf)?;
+
+            CqlValue::Duration(CqlDuration {
+                months,
+                days,
+                nanoseconds,
+            })
+        }
         Inet => CqlValue::Inet(match buf.len() {
             4 => {
                 let ret = IpAddr::from(<[u8; 4]>::try_from(&buf[0..4])?);
@@ -852,7 +890,7 @@ pub fn deserialize(buf: &mut &[u8]) -> StdResult<Result, ParseError> {
 #[cfg(test)]
 mod tests {
     use crate as scylla;
-    use crate::frame::value::Counter;
+    use crate::frame::value::{Counter, CqlDuration};
     use bigdecimal::BigDecimal;
     use chrono::Duration;
     use chrono::NaiveDate;
@@ -1156,13 +1194,11 @@ mod tests {
             super::deser_cql_value(&ColumnType::Date, &mut four_bytes.as_ref()).unwrap();
         assert_eq!(date, CqlValue::Date(u32::from_be_bytes(four_bytes)));
 
-        // Date is parsed as u32 not i32, u32::max_value() is u32::max_value()
-        let date: CqlValue = super::deser_cql_value(
-            &ColumnType::Date,
-            &mut u32::max_value().to_be_bytes().as_ref(),
-        )
-        .unwrap();
-        assert_eq!(date, CqlValue::Date(u32::max_value()));
+        // Date is parsed as u32 not i32, u32::MAX is u32::MAX
+        let date: CqlValue =
+            super::deser_cql_value(&ColumnType::Date, &mut u32::MAX.to_be_bytes().as_ref())
+                .unwrap();
+        assert_eq!(date, CqlValue::Date(u32::MAX));
 
         // Trying to parse a 0, 3 or 5 byte array fails
         super::deser_cql_value(&ColumnType::Date, &mut [].as_ref()).unwrap();
@@ -1197,7 +1233,7 @@ mod tests {
 
         assert_eq!(date.as_date().unwrap(), after_epoch);
 
-        // 0 and u32::max_value() is out of NaiveDate range, fails with an error, not panics
+        // 0 and u32::MAX is out of NaiveDate range, fails with an error, not panics
         assert!(
             super::deser_cql_value(&ColumnType::Date, &mut 0_u32.to_be_bytes().as_ref())
                 .unwrap()
@@ -1205,13 +1241,12 @@ mod tests {
                 .is_none()
         );
 
-        assert!(super::deser_cql_value(
-            &ColumnType::Date,
-            &mut u32::max_value().to_be_bytes().as_ref()
-        )
-        .unwrap()
-        .as_date()
-        .is_none());
+        assert!(
+            super::deser_cql_value(&ColumnType::Date, &mut u32::MAX.to_be_bytes().as_ref())
+                .unwrap()
+                .as_date()
+                .is_none()
+        );
 
         // It's hard to test NaiveDate more because it involves calculating days between calendar dates
         // There are more tests using database queries that should cover it
@@ -1235,7 +1270,7 @@ mod tests {
 
         // Negative values cause an error
         // Values bigger than 86399999999999 cause an error
-        for test_val in [-1, i64::min_value(), max_time + 1, i64::max_value()].iter() {
+        for test_val in [-1, i64::MIN, max_time + 1, i64::MAX].iter() {
             let bytes: [u8; 8] = test_val.to_be_bytes();
             super::deser_cql_value(&ColumnType::Time, &mut &bytes[..]).unwrap_err();
         }
@@ -1259,15 +1294,7 @@ mod tests {
         // Timestamp is an i64 - milliseconds since unix epoch
 
         // Check that test values are deserialized correctly
-        for test_val in &[
-            0,
-            -1,
-            1,
-            74568745,
-            -4584658,
-            i64::min_value(),
-            i64::max_value(),
-        ] {
+        for test_val in &[0, -1, 1, 74568745, -4584658, i64::MIN, i64::MAX] {
             let bytes: [u8; 8] = test_val.to_be_bytes();
             let cql_value: CqlValue =
                 super::deser_cql_value(&ColumnType::Timestamp, &mut &bytes[..]).unwrap();
@@ -1295,6 +1322,21 @@ mod tests {
         assert_eq!(v, vec![0, 0, 0, 0]);
     }
 
+    #[test]
+    fn test_duration_deserialize() {
+        let bytes = [0xc, 0x12, 0xe2, 0x8c, 0x39, 0xd2];
+        let cql_value: CqlValue =
+            super::deser_cql_value(&ColumnType::Duration, &mut &bytes[..]).unwrap();
+        assert_eq!(
+            cql_value,
+            CqlValue::Duration(CqlDuration {
+                months: 6,
+                days: 9,
+                nanoseconds: 21372137
+            })
+        );
+    }
+
     #[test]
     fn test_deserialize_empty_payload() {
         for (test_type, res_cql) in [
