@@ -12,6 +12,7 @@ use thiserror::Error;
 use uuid::Uuid;
 
 use super::response::result::CqlValue;
+use super::types::vint_encode;
 
 /// Every value being sent in a query must implement this trait
 /// serialize() should write the Value as [bytes] to the provided buffer
@@ -60,6 +61,14 @@ pub struct SerializedValues {
     contains_names: bool,
 }
 
+/// Represents a CQL Duration value
+#[derive(Clone, Debug, Copy, PartialEq)]
+pub struct CqlDuration {
+    pub months: i32,
+    pub days: i32,
+    pub nanoseconds: i64,
+}
+
 #[derive(Debug, Error, Clone, Copy, PartialEq, Eq, PartialOrd, Ord)]
 pub enum SerializeValuesError {
     #[error("Too many values to add, max 32 767 values can be sent in a request")]
@@ -134,7 +143,7 @@ impl SerializedValues {
         if self.contains_names {
             return Err(SerializeValuesError::MixingNamedAndNotNamedValues);
         }
-        if self.values_num == i16::max_value() {
+        if self.values_num == i16::MAX {
             return Err(SerializeValuesError::TooManyValues);
         }
 
@@ -158,7 +167,7 @@ impl SerializedValues {
             return Err(SerializeValuesError::MixingNamedAndNotNamedValues);
         }
         self.contains_names = true;
-        if self.values_num == i16::max_value() {
+        if self.values_num == i16::MAX {
             return Err(SerializeValuesError::TooManyValues);
         }
 
@@ -437,6 +446,23 @@ impl Value for Counter {
     }
 }
 
+impl Value for CqlDuration {
+    fn serialize(&self, buf: &mut Vec<u8>) -> Result<(), ValueTooBig> {
+        let bytes_num_pos: usize = buf.len();
+        buf.put_i32(0);
+
+        vint_encode(self.months as i64, buf);
+        vint_encode(self.days as i64, buf);
+        vint_encode(self.nanoseconds, buf);
+
+        let written_bytes: usize = buf.len() - bytes_num_pos - 4;
+        let written_bytes_i32: i32 = written_bytes.try_into().map_err(|_| ValueTooBig)?;
+        buf[bytes_num_pos..(bytes_num_pos + 4)].copy_from_slice(&written_bytes_i32.to_be_bytes());
+
+        Ok(())
+    }
+}
+
 impl<V: Value> Value for MaybeUnset<V> {
     fn serialize(&self, buf: &mut Vec<u8>) -> Result<(), ValueTooBig> {
         match self {
@@ -566,6 +592,7 @@ impl Value for CqlValue {
             }
 
             CqlValue::Date(d) => Date(*d).serialize(buf),
+            CqlValue::Duration(d) => d.serialize(buf),
             CqlValue::Timestamp(t) => Timestamp(*t).serialize(buf),
             CqlValue::Time(t) => Time(*t).serialize(buf),
 
